@@ -2,6 +2,7 @@
 """
 Providing utilities for the workflow package.
 """
+import logging
 import math
 import warnings
 import os
@@ -12,14 +13,22 @@ import torch
 import collections
 from collections import defaultdict, OrderedDict
 import shutil
+from openbabel import openbabel as ob
 from openbabel import pybel
 from tqdm import tqdm
+import numpy as np
 from io import StringIO
 from rdkit import Chem
 from rdkit.Chem.rdMolDescriptors import CalcNumAtomStereoCenters
 from rdkit.Chem.rdMolDescriptors import CalcNumUnspecifiedAtomStereoCenters
+from rdkit.Chem import rdMolDescriptors
 
+#CODATA 2018 energy conversion factor
+hartree2ev = 27.211386245988
+hartree2kcalpermol = 627.50947337481
+ev2kcalpermol = 23.060547830619026
 
+logger = logging.getLogger("auto3d")
 def guess_file_type(filename):
     """Returns the extension for the filename"""
     assert '.' in filename
@@ -37,6 +46,10 @@ def check_input(args):
         each SMILES in the input file.
     """
     print("Checking input file...")
+    logger.info("Checking input file...")
+    # logger.info("================================================================================")
+    # logger.info("                               Check Input")
+    # logger.info("================================================================================")
     ANI_elements = {1, 6, 7, 8, 9, 16, 17}
     ANI = True
     # Check --use_gpu
@@ -45,11 +58,11 @@ def check_input(args):
         if torch.cuda.is_available() == False:
             sys.exit("No cuda device was detected. Please set --use_gpu=False.")
     # Check the availability of omega
-    if "OE_LICENSE" not in os.environ:
-        warnings.warn("OpenEye software license is not detected. Please use RDKit for your program where applicable.")
-        isomer_engine = args.isomer_engine
-        if isomer_engine == "omega":
-            sys.exit("Omega is used as isomer engine, but OE_LICENSE is not detected.")
+    # if "OE_LICENSE" not in os.environ:
+    #     warnings.warn("OpenEye software license is not detected. Please use RDKit for your program where applicable.")
+    isomer_engine = args.isomer_engine
+    if ("OE_LICENSE" not in os.environ) and (isomer_engine == "omega"):
+        sys.exit("Omega is used as the isomer engine, but OE_LICENSE is not detected. Please use rdkit.")
 
     # Check the installation for open toolkits, torchani
     if args.isomer_engine == "omega":
@@ -88,6 +101,7 @@ def check_input(args):
         smiles_all.append(smiles)
     print(f"\tThere are {len(data)} SMILES in the input file {args.path}. ")
     print("\tAll SMILES and IDs are valid.")
+    logger.info(f"\tThere are {len(data)} SMILES in the input file {args.path}. \n\tAll SMILES and IDs are valid.")
 
     # Check number of unspecified atomic stereo center
     if args.enumerate_isomer == False:
@@ -108,15 +122,147 @@ def check_input(args):
             only_aimnet_smiles.append(smiles)
 
     print("Suggestions for choosing isomer_engine and optimizing_engine: ")
+    logger.info(f"Suggestions for choosing isomer_engine and optimizing_engine: ")
     if ANI:
         print("\tIsomer engine options: RDKit and Omega.\n"
-              "\tOptimizing engine options: ANI2x, ANI_2xt and AIMNET.")
+              "\tOptimizing engine options: ANI2x, ANI2xt and AIMNET.")
+        logger.info("\tIsomer engine options: RDKit and Omega.")
+        logger.info("\tOptimizing engine options: ANI2x, ANI2xt and AIMNET.")
     else:
         print("\tIsomer engine options: RDKit and Omega.\n"
               "\tOptimizing engine options: AIMNET.")
+        logger.info("\tIsomer engine options: RDKit and Omega.")
+        logger.info("\tOptimizing engine options: AIMNET.")
         optimizing_engine = args.optimizing_engine
         if optimizing_engine != "AIMNET":
             sys.exit(f"Only AIMNET can handle: {only_aimnet_smiles}, but {optimizing_engine} was parsed to Auto3D.")
+            logger.critical(f"Only AIMNET can handle: {only_aimnet_smiles}, but {optimizing_engine} was parsed to Auto3D.")
+
+def to_smiles(path, fomat="sdf"):
+    """converting a file from a given format to smi file
+    input: path
+    format: [optional] sdf
+    
+    returns: a path of smi file containing the same molecules as in the sdf"""
+    suppl = Chem.SDMolSupplier(path)
+    smiles = []
+    for i, mol in enumerate(suppl):
+        name = mol.GetProp("_Name").strip()
+        if len(name) == 0:  #len("") == 0
+            name = str(i)
+        smiles.append((Chem.MolToSmiles(mol, isomericSmiles=True, canonical=True), name))
+    
+    #write
+    folder = os.path.dirname(os.path.abspath(path))
+    new_base = os.path.basename(path).split(".")[0].strip() + ".smi"
+    new_path = os.path.join(folder, new_base)
+    with open(new_path, "w") as f:
+        for smi, name in smiles:
+            f.write(f"{smi} {name}\n")
+    return new_path
+
+
+def report(path):
+    """Given a smi file, reports the following:
+    - number of SMILES
+    - SMILES size distribution
+    - element type and percent of molecules with this types
+    - number of charged molecules
+    - number of SMILES with unspecified stereo center
+    """
+    suppl = Chem.SmilesMolSupplier(path, titleLine=False)
+    c = 0  #count number of SMILES
+    sizes = []
+    element_counts = defaultdict(lambda: 0)
+    charges = []
+    charge_counts = defaultdict(lambda: 0)
+    num_charged_mols = 0
+    unspecified_atom_centers = []
+    num_unspecified_mols = 0
+    for mol in suppl:
+        c += 1
+        atoms = [a.GetAtomicNum() for a in mol.GetAtoms()]  #H not included
+        elements = list(set(atoms))
+        sizes.append(len(atoms))
+        for e in elements:
+            element_counts[e] += 1
+        
+        charge = Chem.rdmolops.GetFormalCharge(mol)
+        charge_counts[charge] += 1
+        charges.append(charge)
+        if charge != 0:
+            num_charged_mols += 1
+        
+        unspecified_centers = CalcNumUnspecifiedAtomStereoCenters(mol)
+        unspecified_atom_centers.append(unspecified_centers)
+        if unspecified_centers > 0:
+            num_unspecified_mols += 1
+    
+    print("Total number of SMILES: ", c)
+    print(f"SMILES size distribution: mean={str(np.mean(sizes))} std={str(np.std(sizes))} min={str(min(sizes))} max={str(max(sizes))}")    
+    print("Breakdown of element types and its prevailance: ")
+    for e, c_e in sorted(element_counts.items()):
+        print(f"    {str(e)} total: {str(c_e)}  percent: {str(round(c_e/c, 3))}")
+    print(f"Number of charged molecules: {str(num_charged_mols)}")
+    print("Breakdown of charge distribution")
+    for charge, c_c in sorted(charge_counts.items()):
+        print(f"    charge={str(charge)} total: {str(c_c)} percent: {str(round(c_c/c, 3))}")
+    print(f"Number of molecules with unspecified atomic centers: {str(num_unspecified_mols)}")
+
+
+def replace_(name, new="-"):
+    """
+    replace the underscore in a string with user specified character
+    name: a string
+    new: a tring, lenth=1, used to replace the underscore
+    """
+    new_name = ""
+    for letter in name:
+        if letter == "_":
+            new_name += new
+        else:
+            new_name += letter
+    return new_name
+
+
+def find_smiles_not_in_sdf(smi, sdf):
+    """Find the SMILES who doesn't have a 3D structure in the SDF file
+    smi: path to an smi file (the input path for Auto3D)
+    sdf: path to an SDF file"""
+    #find all SMILES ids
+    smi_names = []
+    with open(smi, "r") as f:
+        data = f.readlines()
+    for line in data:
+        smi, id = tuple(line.strip().split())
+        smi_names.append((smi.strip(), id.strip()))
+    
+    #find the titles of the SDF file
+    # with open(sdf, "r") as f:
+    #     sdf_data = f.readlines()
+    sdf_data = []
+    mols = Chem.SDMolSupplier(sdf)
+    for mol in mols:
+        sdf_data.append(mol.GetProp("_Name"))
+    sdf_data = list(set(sdf_data))
+
+    bad = []
+    for smi, id in smi_names:
+        has_3D_structure = False
+        for line in sdf_data:
+            if id in line:
+                has_3D_structure = True
+        if not has_3D_structure:
+            bad.append((id, smi))
+
+    if len(bad) > 0:
+        print("The following SMILES has no 3D structure in the SDF file.")
+        print("ID, SMILES")
+        for id, smi in bad:
+            print(id, smi)
+    else:
+        print("Every SMILES has at least an 3D structure in the SDF file.")
+    return bad
 
 
 class NullIO(StringIO):
@@ -184,8 +330,8 @@ def hash_taut_smi(smi, out):
         smiles, id = line.strip().split()
         c = 1
         id_ = id
-        while (id_ in dict0.keys()):
-            id_ = id + f"taut{c}"
+        while (('taut' not in id_) or (id_ in dict0.keys())):
+            id_ = id + f"@taut{c}"
             c += 1
         dict0[id_] = smiles
 
@@ -399,6 +545,8 @@ def remove_enantiomers(inpath, out):
         except:
             new_values = values
             print(f"Enantiomers not removed for {key}")
+            logger.info(f"Enantiomers not removed for {key}")
+            
         smiles[key] = new_values
         
     with open(out, 'w+') as f:
@@ -409,6 +557,32 @@ def remove_enantiomers(inpath, out):
                 f.write(line)
     return smiles
 
+def atomType(mol, atomIdx):
+    """get the atomic type given an atom index, both in pybel mol object"""
+    atom_num = mol.OBMol.GetAtom(atomIdx).GetAtomicNum()
+    return atom_num
+
+
+def check_bonds(mol):
+    """Check if a pybel mol object has valid bond lengths"""
+    # Initialize UFF bond radii (Rappe et al. JACS 1992)
+    # Units of angstroms 
+    # These radii neglect the bond-order and electronegativity corrections in the original paper. Where several values exist for the same atom, the largest was used. 
+    Radii = {1:0.354, 
+             5:0.838, 6:0.757, 7:0.700,  8:0.658,  9:0.668,
+             14:1.117, 15:1.117, 16:1.064, 17:1.044,
+             32: 1.197, 33:1.211, 34:1.190, 35:1.192,
+             51:1.407, 52:1.386,  53:1.382}
+
+    for bond in ob.OBMolBondIter(mol.OBMol):
+        length = bond.GetLength()
+        begin = atomType(mol, bond.GetBeginAtomIdx())
+        end = atomType(mol, bond.GetEndAtomIdx())
+        reference_length = (Radii[begin] + Radii[end]) * 1.25
+        if length > reference_length:
+            return False
+    return True
+
 
 def filter_unique(mols, crit=0.3):
     """Remove structures that are very similar.
@@ -424,7 +598,8 @@ def filter_unique(mols, crit=0.3):
     mols_ = []
     for mol in mols:
         convergence_flag = str(mol.data['Converged']).lower() == "true"
-        if convergence_flag:
+        has_valid_bonds = check_bonds(mol)
+        if convergence_flag and has_valid_bonds:
             mols_.append(mol)
     mols = mols_
 
@@ -631,6 +806,7 @@ def amend_configuration(smis):
                 dct[key] = value
             except:
                 print(f"Stereo centers for {key} are not fully enumerated.")
+                logger.info(f"Stereo centers for {key} are not fully enumerated.")
     return dct
 
 def amend_configuration_w(smi):
